/*
 * @Author: ysh
 * @Date: 2025-07-08 15:10:42
 * @LastEditors: Please set LastEditors
 * @LastEditTime: 2025-07-09 11:39:18
 */
package com.ruoyi.web.llm.service.impl;

import com.ruoyi.common.config.RuoYiConfig;
import com.ruoyi.web.llm.service.ILangChainMilvusService;
import dev.langchain4j.data.document.Document;
import dev.langchain4j.data.document.parser.apache.pdfbox.ApachePdfBoxDocumentParser;
import dev.langchain4j.data.document.splitter.DocumentByParagraphSplitter;
import dev.langchain4j.data.segment.TextSegment;
import dev.langchain4j.model.embedding.EmbeddingModel;

import io.milvus.client.MilvusClient;
import io.milvus.grpc.MutationResult;
import io.milvus.grpc.SearchResults;
import io.milvus.param.MetricType;
import io.milvus.param.R;
import io.milvus.param.RpcStatus;
import io.milvus.param.collection.LoadCollectionParam;
import io.milvus.param.collection.ReleaseCollectionParam;
import io.milvus.param.dml.InsertParam;
import io.milvus.param.dml.SearchParam;
import io.milvus.response.SearchResultsWrapper;
import org.apache.poi.xwpf.extractor.XWPFWordExtractor;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.noear.solon.ai.chat.ChatModel;
import org.noear.solon.ai.chat.ChatResponse;
import org.noear.solon.ai.chat.message.AssistantMessage;
import org.reactivestreams.Publisher;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import reactor.core.publisher.Flux;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class LangChainMilvusServiceImpl implements ILangChainMilvusService
{
    /**
     * 导入知识库文件
     */
    @Override
    public R<MutationResult> insertLangchainEmbeddingDocument(MilvusClient milvusClient, MultipartFile file, String collectionName, EmbeddingModel embeddingModel) throws IOException
    {
        File profilePath = new File( RuoYiConfig.getProfile() + "/upload/knowledge");
        if (!profilePath.exists())
            profilePath.mkdirs();
        File transferFile = new File( profilePath + File.separator + file.getOriginalFilename());
        if (!transferFile.exists()) {
            file.transferTo(transferFile);
        }
        // 加载文档
        Document document;
        InputStream fileInputStream = new FileInputStream(transferFile);
        String filename = file.getOriginalFilename().toLowerCase();
        if (filename.endsWith(".docx")) {
            XWPFDocument docx = new XWPFDocument(fileInputStream);
            XWPFWordExtractor extractor = new XWPFWordExtractor(docx);
            String text = extractor.getText();
            document = Document.from(text);
        }
        else if (filename.endsWith(".pdf")) {
            document = new ApachePdfBoxDocumentParser().parse(fileInputStream);
        }
        else {
            throw new UnsupportedOperationException("不支持文件类型: " + filename);
        }
        DocumentByParagraphSplitter splitter = new DocumentByParagraphSplitter(1024,0);;
        List<TextSegment> segments = splitter.split(document);

        // 提取文本和生成嵌入
        List<String> fileNames = new ArrayList<>();
        List<String> fileTypes = new ArrayList<>();
        List<String> texts = new ArrayList<>();
        List<List<Float>> embeddings = new ArrayList<>();

        for (TextSegment segment : segments) {
            String text = segment.text();
            if (text.trim().isEmpty())
                continue;
            fileNames.add(file.getOriginalFilename());
            String[] fileName = file.getOriginalFilename().split("\\.");
            fileTypes.add(fileName[fileName.length - 1]);
            texts.add(text);
            embeddings.add(embeddingModel.embed(text).content().vectorAsList());
        }

        // 准备导入数据
        List<InsertParam.Field> fields = new ArrayList<>();
        fields.add(new InsertParam.Field("file_name", fileNames));
        fields.add(new InsertParam.Field("file_type", fileTypes));
        fields.add(new InsertParam.Field("content", texts));
        fields.add(new InsertParam.Field("embedding", embeddings));

        InsertParam insertParam = InsertParam.newBuilder()
                .withCollectionName(collectionName)
                .withFields(fields)
                .build();

        // 执行导入
        return milvusClient.insert(insertParam);
    }

    /**
     * 从Milvus检索相关文档
     */
    @Override
    public List<String> retrieveFromMilvus(MilvusClient milvusClient, EmbeddingModel embeddingModel, String collectionName, String query, int topK) {
        List<List<Float>> queryVector = Collections.singletonList(embeddingModel.embed(query).content().vectorAsList());

        //  加载集合
        LoadCollectionParam loadParam = LoadCollectionParam.newBuilder()
                .withCollectionName(collectionName)
                .build();

        R<RpcStatus> loadResponse = milvusClient.loadCollection(loadParam);
        if (loadResponse.getStatus() != R.Status.Success.getCode()) {
            System.err.println("加载Collection失败: " + loadResponse.getMessage());
            milvusClient.close();
        }

        // 构建SearchParam
        SearchParam searchParam = SearchParam.newBuilder()
                .withCollectionName(collectionName)
                .withVectors(queryVector)
                .withTopK(topK)
                .withOutFields(Arrays.asList("file_name", "file_type", "content"))
                .withVectorFieldName("embedding")
                .withMetricType(MetricType.COSINE)
                .withParams("{\"nprobe\": 1}")
                .build();

        R<SearchResults> response = milvusClient.search(searchParam);
        SearchResultsWrapper wrapper = new SearchResultsWrapper(response.getData().getResults());

        // 释放集合
        ReleaseCollectionParam param = ReleaseCollectionParam.newBuilder()
                .withCollectionName(collectionName)
                .build();
        milvusClient.releaseCollection(param);

        return wrapper.getRowRecords(0).stream()
                .map(record -> (String) record.get("content"))
                .collect(Collectors.toList());
    }

    /**
     * 调用LLM+RAG生成回答
     * @return
     */
    @Override
    public Flux<AssistantMessage> generateAnswerWithRag(String question, List<String> contexts, String llmServiceUrl) {
        StringBuilder sb = new StringBuilder();
        sb.append("根据以下上下文回答问题：\n\n");
        for (int i = 0; i < contexts.size(); i++) {
            sb.append("上下文").append(i+1).append(": ").append(contexts.get(i)).append("\n\n");
        }
        sb.append("问题: ").append(question).append("\n回答: ");
        // 构建带动态参数的URL
        return generateAnswer(sb.toString(),llmServiceUrl);
    }

    /**
     * 调用LLM生成回答
     * @return
     */
    @Override
    public Flux<AssistantMessage> generateAnswer(String prompt, String llmServiceUrl) {
        ChatModel chatModel = ChatModel.of(llmServiceUrl)
                .provider("openai")
                .model("DeepSeek-R1-Distill-Qwen-1.5B")
                .apiKey("1")
                .build();
        Publisher<ChatResponse> publisher = chatModel.prompt(prompt).stream();

        return Flux.from(publisher)
                .map(response -> {
                    return response.getChoices().get(0).getMessage();
                });
    }

}